Index (P)
»
BLS
»
PointError
Exception: BLS::PointError
Inherits:
Error
Object
StandardError
Error
BLS::PointError
show all
Defined in:
lib/bls/error.rb
Overview
A point that could not be parsed, is not on the curve, or is not in the prime-order subgroup.